<?php

namespace lib;

use lib\help\Arr;

/**
 * Class Db
 * @package lib
 */
class Db extends Model
{

    /**
     * 页码
     * @var int
     */
    protected $page;

    /**
     *
     * @var int
     */
    protected $limit;

    /**
     * 排序
     * @var string|array
     */
    protected $order;

    /**
     * 分组
     * @var string
     */
    protected $group;

    /**
     * 表别名
     * @var string
     */
    protected $alera;

    /**
     * 表名
     * @var string
     */
    protected $table;

    /**
     * 查询条件
     * @var array
     */
    protected $where = [];

    /**
     * 查询字段
     * @var string|array
     */
    protected $field;

    /**
     * sql语句
     * @var string
     */
    protected $sql;

    /**
     * @param $field
     * @return $this
     */
    public function field($field)
    {
        if (is_array($field)) {
            $this->field = Arr::fieldArray($field, ',');
        } else {
            $this->field = $field;
        }
        return $this;
    }

    /**
     * @param int $page
     * @param int $limit
     * @return $this
     */
    public function page(int $page = 1, int $limit = 10)
    {
        $this->page = $page;
        $this->limit = $limit;
        return $this;
    }

    /**
     * @param int $limit
     * @return $this
     */
    public function limit(int $limit = 10)
    {
        $this->limit = $limit;
        return $this;
    }

    /**
     * @param $where
     * @return $this
     */
    public function where($where, $exp = null, $value = null)
    {
        if (is_string($where) && $exp && is_null($value)) {
            $this->where[$where] = $exp;
        } elseif (is_string($where) && $exp && !is_null($value)) {
            $this->where[$where] = [$exp => $value];
        } elseif (is_array($where)) {
            $this->where = array_merge($this->where, $where);
        }
        return $this;
    }

    /**
     * @param string $table
     * @return static
     */
    public static function table(string $table)
    {
        self::instance()->table = $table;
        return self::instance();
    }

    /**
     * @param string $name
     * @return static
     */
    public static function name(string $name)
    {
        self::instance()->table = self::instance()->database['prefix'] . $name;
        return self::instance();
    }

    /**
     * @param string $group
     * @return $this
     */
    public function group(string $group)
    {
        $this->group = $group;
        return $this;
    }

    /**
     * @param $order
     * @return $this
     */
    public function order($order)
    {
        $this->order = $order;
        return $this;
    }

    /**
     * @return mixed
     */
    public function find()
    {
        $res = $this->selectSql($this->table, $this->field, $this->where, $this->order, '', 1)->selectFind();
        $this->clera();
        return $res;
    }

    /**
     * @param array $data
     * @param null $pkId
     * @return mixed|void
     */
    public function save(array $data, $pkId = null)
    {
        $table = $this->table;
        if ($pkId) {
            return $this->update($table, $data, is_array($pkId) ? $pkId : [self::pk() => $pkId]);
        } else {
            return $this->insert($this->table, $data);
        }
    }

    public function edit($data,$where = [])
    {
        if($where){
            $this->where($where);
        }
        $re = $this->updateSql($this->table,$data, $this->where);
        $this->clera();
        return $re;
    }

    /**
     * @param array $data
     * @return array|bool|\PDOStatement
     */
    public function saveAll(array $data)
    {
        $res = true;
        foreach ($data as $item) {
            $res = $res && $this->insert($this->table, $item);
        }
        return $res;
    }

    /**
     * @return mixed
     */
    public function select()
    {
        $limit = '';
        if ($this->page && $this->limit) {
            $page = $this->page;
            $limit = ($page - 1) * $this->limit . ',' . $this->limit;
        }
        $list = $this->selectSql($this->table, $this->field, $this->where, $this->order, '', $limit)->selectAll();
        $this->clera();
        return $list;
    }

    protected function clera()
    {
        $this->table = null;
        $this->field = null;
        $this->where = [];
        $this->order = null;
        $this->limit = null;
        $this->page = null;
    }

    /**
     * @param string $sql
     * @param bool $isQuery
     * @return array|bool|\PDOStatement
     */
    public static function querySql(string $sql, bool $isQuery = false)
    {
        $prepare = self::instance()->pdo->query($sql);
        if ($isQuery) {
            return $prepare;
        }
        $prepare->execute();
        return $prepare->fetchAll(\PDO::FETCH_ASSOC);
    }

    /**
     * @param string $sql
     * @param string $field
     * @return string
     */
    public static function queryFind(string $sql, string $field)
    {
        $prepare = self::instance()->pdo->query($sql);
        $prepare->execute();
        $list = $prepare->fetch(\PDO::FETCH_ASSOC);
        return $list[$field] ?? '';
    }

    /**
     * 获取主键
     * @param string $name
     * @return string
     */
    public static function pk()
    {
        $fileData = self::querySql('desc ' . self::instance()->table);
        $idFirld = '';
        foreach ($fileData as $item) {
            if (isset($item['Extra']) && $item['Extra'] && strtolower($item['Extra']) == 'auto_increment') {
                $idFirld = $item['Field'];
                break;
            }
        }
        self::instance()->table = null;
        return $idFirld;
    }

    /**
     * 检测字段是否存在
     * @param string $name
     * @param string $filed
     * @return bool
     */
    public static function be(string $filed)
    {
        $fileData = self::querySql('desc ' . self::instance()->table);
        $isBe = false;
        foreach ($fileData as $item) {
            if (isset($item['Field']) && $item['Field'] && strtolower($item['Field']) == strtolower($filed)) {
                $isBe = true;
                break;
            }
        }
        self::instance()->table = null;
        return $isBe;
    }

    /**
     * 表是否存在
     * @param string $name
     * @return bool
     */
    public static function isTabel()
    {
        return self::querySql('SHOW TABLES LIKE "' . self::instance()->table . '"') ? true : false;
    }

}