<?php

namespace lib;

use lib\help\Arr;
use lib\help\Config;
use lib\trail\Instance;

/**
 * Class Request
 * @package lib
 */
class Request
{

    use Instance;

    /**
     * @var array
     */
    protected $get;

    /**
     * @var array
     */
    protected $post;

    /**
     * @var string
     */
    protected $action;

    /**
     * @var string
     */
    protected $controller;

    /**
     * @var array
     */
    protected $version;

    /**
     * @var array
     */
    protected $upgradeData = [
        'table' => '',
        'sql' => '',
        'type' => 1,
        'where' => [],
        'field' => '',
        'info' => '',
    ];

    /**
     * @var string
     */
    protected $upgradeFile;

    protected function __construct()
    {
        $this->get = $_GET;
        $this->post = $_POST;
        $this->init();
    }

    protected function init()
    {
        $this->controller = Arr::getArrayValue($this->get, 'c', 'Index');
        $this->action = Arr::getArrayValue($this->get, 'a', 'index');
        $this->version = parse_ini_file(Config::get('versionDir'));
        $this->upgradeFile = Config::get('upgradeDie');
    }

    /**
     * 获取当前请求控制器
     * @return mixed
     */
    public function controller()
    {
        return $this->controller;
    }

    /**
     * 获取当前请求方法
     * @return mixed
     */
    public function action()
    {
        return $this->action;
    }

    /**
     * @return array
     */
    public function getUpgradeDataDefault()
    {
        return $this->upgradeData;
    }


    /**
     * 获取get数据
     * @param null $name
     * @param null $default
     * @param bool $suffix
     * @return array|mixed
     */
    public function getData($name = null, $default = null, bool $suffix = false)
    {
        return Arr::getArrayValue($this->get, $name, $default, $suffix);
    }

    /**
     * 获取post数据
     * @param null $name
     * @param null $default
     * @param bool $suffix
     * @return array|mixed
     */
    public function postData($name = null, $default = null, bool $suffix = false)
    {
        return Arr::getArrayValue($this->post, $name, $default, $suffix);
    }

    /**
     * 获取升级数据
     * @return mixed
     */
    public function getUpgradeData()
    {
        $data = include $this->upgradeFile;
        foreach ($data as &$item) {
            $item = array_merge($this->upgradeData, $item);
        }
        return $data;
    }

    public function cleanUpgradeData()
    {
        file_put_contents($this->upgradeFile, "<?php\r\n return [];");
    }

    /**
     * @return string
     */
    public function version()
    {
        return $this->version['version'] ?? 'v1.0';
    }

    /**
     * @return bool
     */
    public function isUpgrade()
    {
        return ($this->version['upgrade'] ?? false) ? true : false;
    }

    /**
     * 设置版本
     */
    public function setVersion()
    {
        $version = $this->version['version'];
        $version_code = $this->version['version_code'];
        $content = <<<CONTENT
version={$version}
version_code={$version_code}
upgrade=1
CONTENT;
        file_put_contents(Config::get('versionDir'), $content);
    }

    /**
     * @param string $field
     * @param int $n
     * @return bool
     */
    public function setIsUpgrade(string $field, int $n = 0)
    {
        $upgrade = parse_ini_file(RUN_DIR . '/.upgrade');
        if ($n) {
            if (!is_array($upgrade)) {
                $upgrade = [];
            }
            $string = '';
            foreach ($upgrade as $key => $item) {
                $string .= $key . '=' . $item . "\r\n";
            }
            $string .= $field . '=' . $n . "\r\n";
            file_put_contents(RUN_DIR . '/.upgrade', $string);
            return true;
        } else {
            if (!is_array($upgrade)) {
                return false;
            }
            return isset($upgrade[$field]);
        }
    }
}